{{
*****************************************************************************
* One Wire Data Collection Process
* This reads each of 1-wire sensors and then loads the base data.
* We do some of the basic statistics, but we don't do much.
* that's left to the stat engine.
*
* This is based on a lot of peoples code, Cam Thompson, Micah Dowty, Jon McPhalen
* and Wookies post. 
*****************************************************************************

}}

CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  
  DEBUG             = 1
  OW_DATA           = 0                                 ' 1-wire data pin
  SKIP_ROM          = $CC                               ' 1-wire commands
  READ_SCRATCHPAD   = $BE
  CONVERT_T         = $44
  MATCH_ROM         = $55
  SEARCH_ROM        = $F0
  READ_ROM          = $33
  ALARM_SEARCH      = $EC
  COPY_SCRATCHPAD   = $48
  RECALL_EE         = $B8
  READ_POWER_SUPPLY = $B4

  ' 1-wire family codes
  FAMILY_DS18B20    = $28
  FAMILY_TEMP   = $10 

  ' Search flags
  REQUIRE_CRC       = $100
  CLS               = $00                               ' clear screen
  HOME              = $01                               ' home
  CR                = $0D                               ' carriage return
  DEG               = $B0                               ' degree symbol  
  QUOTE             = 35            

  MAX_DEVICES       = 10                                ' maximum number of 1-wire devices
' WeatherStation
  WINDSPEED = 1
  AIRTEMP = 2
  WINDDIR = 3
  BARO  = 4
  HUMIDITY = 5
  LIGHTNING = 6
  SOLAR = 7
  RAINFALL = 8
  WATERTEMP = 9
  WATERSPEED = 10
  WATERCLEAR = 11
  WATERDIR   = 12
  TIDELEVEL = 13
  SALINITY = 14
  LIGHTLEVEL = 15
  VARCNT = 15


  
OBJ

  PST          : "Parallax Serial Terminal"
  ow            : "OneWire"
  fp            : "FloatString"
  f             : "FloatMath"                           ' could also use Float32

VAR

  long OWaddressList[MAX_DEVICES*2]                      ' 64-bit address buffer
  Word DataVal[20], DataValAvg[20], DataValSamp[20], DataValHrAvg[20], DataValDayAvg[20]
  Word DataValMonthAvg[20],DataValYearAvg[20], DataValLastUpd[20]
  Byte NumDataDevices
  Byte COGDirective[8]
  ' The CogSensor variable is a set of commands.
  ' I realized 1/2 way through, that I needed a better intercog communication process
  ' so I created one
  Byte CogSensor, CogDigital, CogAnalog, CogTransmit, CogDisplay, CogStat
  Byte SemSensor, SemDigital, SemAnalog, SemTransmit, SemDisplay, SemStat
  Long DataPass[4]
  Byte SemDataPass 
  Long OWStack[30]
  byte TrSemID
  Long RevAddr[2]
 

  
PUB OWMain | rcode, PCogCmd
' this is the routine that is kicked off by "Cognew"
' the value in CogSensor tells the program what to do, including shut down.
' We will occasionally refresh the one wire devices just to report status.
' This was a colossal pain so anyone with improvements, bring it on.

' Setup here

    PST.start(115_200)                                    ' Start PST
    waitcnt(clkfreq * 2 + cnt)                            ' let settle down
   CogSensor := 0
   if (SemSensor := LockNew) == -1
     pst.str(string("no locks available"))
   else
      rcode := lockclr(SemSensor)


  Repeat
    if cogSensor == 0                                  ' refresh 
        FindDevices
        Cogsensor := 1                                    ' Find the devices on the bus 
    if CogSensor == 1                                  ' Go to normal 
        cogStat := 0                                   ' normal processing
        LockSet(SemSensor)                             ' stop any updates
        LoopandUpdate                                  ' Read all OW sensors and update data
        lockClr(SemSensor)                             ' Allow others to update
        CogStat := 1                                   ' Kick the stat cog into execute. 
    if CogSensor == 2                                  ' Stop the cog (shutdown)
        Cogstop(cogid)                                 ' stop this cog   
    if Cogsensor == 3                                  ' Loop until changed
        Waitcnt(clkfreq/10 + cnt)                      ' 10th second loop
    waitcnt(clkfreq * 2 + cnt)

    
PUB FindDevices | n, i, p
  ' read all the OW devices.
  ' at the end we should have
  ' NumDataDevices which is a count of the actual ow devices on the bus
  ' OWaddressList is the list of OW devices by their 64 bit serial number.
  ' We only validate that they are there, we don't read them (yet) 
  pst.str(string("1-Wire Search Demo",13))                 ' Who are we
  longfill(@OWaddresslist,0,MAX_DEVICES * 2)              ' Zero out the addresses

  ' go find all the OW devices
  ow.start(OW_DATA)                                     ' start 1-wire object, pin 0
  NumDataDevices := ow.search(0, MAX_DEVICES, @OWaddressList)          ' search the 1-wire network
  PST.dec(NumDataDevices)
  pst.str(string(" Devices were found.",13))     
  waitcnt(clkfreq * 2 + cnt)     


  ' Run through the list and display all the devices we found
  PST.str(string(13, "Device Address"))                ' display each device        
  p := @OWAddressList
  repeat NumDataDevices 
    PST.Char(CR)
    ' this is more important than it might seem. It affects the conversion routines
    case byte[p]                                        ' display family name
      $01:    PST.str(@ds2401_name)
      $05:    PST.str(@ds2405_name)
      $10:    PST.str(@ds1820_name)
      $22:    PST.str(@ds1822_name)
      $1D:    pst.str(@ds2423_name)                  
      other:  PST.str(@unknown_name)
      ' bytes come out in the same order they went in. 
    repeat i from 0 to 7
      pst.hex(byte[p+i],2)
                                           ' display 64-bit address
    if ow.crc8(8, p) <> 0                               ' check crc of address
      PST.str(string("?crc"))
    p += 8
  pst.str(string(13,"End 1-Wire Search. ",13,13))                        ' get the next CRC

Pub LoopandUpdate | i, addr, tempVal, p , pstart
  ' this makes one complete pass through all of the ow sensors
  ' it is intended to scan them all. But once we have a hit, update
  ' I needed a more elegant way to do this, so any suggestions, would be good.

{{
I have 2 OW devices on my buss
DS1820 104FF5E5010800CB - This is the AirTemperature
DS2423 1D2EC70F0000008F - This is windspeed

}}

  pst.str(string("Starting to Loop through the devices on the buss",13))
  
  p := @OWAddressList
  repeat i from 0 to NumDataDevices -1
      pstart := p+i
      addr:=@OWAddressList + (i*2)  
      tempVal:=GetData(addr)
      ' this is the DS1820 with reverse byte. But I never get here.
      
      If Long[pstart] == $E5F54F10 and Long[pstart+1] == $CB000801
          pst.str(string(13,"Yes"))
          DataVal[AIRTEMP] := CvtTemp(TempVal)
          DataValSamp[AIRTEMP]++
    if ow.crc8(8, p) <> 0                               ' check crc of address
      PST.str(string("?crc"))
    p += 8

     
Pub GetData(addr) | temp, TimeOut
  '' Read the data
  '' this is not working quite the way I expected.
  '' Addr is the address of the bytes in original form
  '' but they are lsb
  '' RevAddr is the reverse of that.
{{  
  RevAddr := 0
  '' reverse bytes. 
  RevAddr[0] := byte[Addr+0]<<24 + byte[Addr+1]<<16 + byte[Addr+2] << 8 + byte[Addr+3]
  RevAddr[1] := byte[Addr+4]<<24 + byte[Addr+5]<<16 + byte[Addr+6] << 8 + byte[Addr+7]  
  pst.hex(RevAddr[0],8)
  pst.hex(RevAddr[1],8)
}}  
  ow.reset                                              ' Rest bus
  ow.writeByte(MATCH_ROM)                               ' match the 64 bit address
  ' do I use the Addr or RevAdder ? Tried a both without success
  ' left it at Addr
  
'  ow.writeAddress(RevAddr)                                 ' write it
  ow.writeAddress(addr)                                 ' write it       
  ow.writeByte(CONVERT_T)                               ' Gotta find out what this does

  ' Wait for the conversion
  Timeout := 0 
  repeat
    waitcnt(clkfreq/100 + cnt)                          ' If we don't get a response in 4 seconds
    Timeout++                                           ' then we died
    if Timeout > 400                                    ' and we need to end
      pst.str(string(13,"Failed to connect to "))
      pst.Hex(addr[0],8)
      pst.hex(addr[1],8)
      waitcnt(clkfreq * 2 + cnt)                                        ' post error
      Result := 0                                       ' Right now
    else
      if ow.readBits(1)
      ' Have a reading! Read it from the scratchpad.

        ow.reset
        ow.writeByte(MATCH_ROM)
        ow.writeAddress(Addr)

        ow.writeByte(READ_SCRATCHPAD)
        Result := ow.readBits(16)
        

Pub CvtTemp(Temp) | DegC, DegF
      ' Convert from fixed point to floating point
      degC := f.FDiv(f.FFloat(temp), 16.0)
     ' Convert celsius to fahrenheit
      degF := f.FAdd(f.FMul(degC, 1.8), 32.0)
       fp.SetPrecision(4)
      'pst.str(f.toStr(degF,NUM#DEC2))
      pst.str(string("F    "))

      return

DAT
ds2401_name     byte    "DS2401 ", 0
ds2405_name     byte    "DS2405 ", 0 
ds1820_name     byte    "DS1820 ", 0 
ds1822_name     byte    "DS1822 ", 0 
unknown_name    byte    "------ ", 0
ds2423_name     byte    "DS2423 ", 0


{{

This is the PST output with addr
--------------------------------------------------------------
1-Wire Search Demo
2 Devices were found.

Device Address
DS1820 104FF5E5010800CB
DS2423 1D2EC70F0000008F
End 1-Wire Search.

Starting to Loop through the devices on the buss

Failed to connect to 000010D800000001
Failed to connect to 000010D800000001
Failed to connect to 000010D800000001
Failed to connect to 000010D800000001
------------------------------------------------------------------------
Same with RevAddr
-----------------------------------------------------------------------
1-Wire Search Demo
2 Devices were found.

Device Address
DS1820 104FF5E5010800CB
DS2423 1D2EC70F0000008F
End 1-Wire Search.

Starting to Loop through the devices on the buss
104FF5E5010800CB
Failed to connect to 104FF5E5010800CB
Failed to connect to 104FF5E5010800CB
Failed to connect to 104FF5E5010800CB



}}